package server

import (
	"acme-mana/src/conf"
	"acme-mana/src/server/handle"
	"context"
	"errors"
	"github.com/gin-gonic/gin"
	"log"
	"net/http"
	"strconv"
	"time"
)

type HttpServer struct {
	server *http.Server `json:"server"`
	status bool
	engine *gin.Engine
}

func (s *HttpServer) InitServer() {
	s.engine = gin.Default()
	s.register()
	s.status = false

	config := conf.Config()
	var serverConf = config.Server
	s.server = &http.Server{
		Addr:    serverConf.Host + ":" + strconv.Itoa(serverConf.Port),
		Handler: s.engine,
	}
}

func (s *HttpServer) Start() {
	go func() {
		if err := s.server.ListenAndServe(); !errors.Is(err, http.ErrServerClosed) {
			log.Printf("listen: %s\n\n", err)
			s.status = true
		} else {
			s.status = false
		}
	}()
}

func (s *HttpServer) Stop() {
	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
	defer cancel()
	err := httpServer.server.Shutdown(ctx)
	s.status = false
	if err != nil {
		log.Fatalln("Server Shutdown:", err)
	}
}

func Status() bool {
	return httpServer.status
}

func (s *HttpServer) register() {
	service := s.engine
	service.GET("/api/v1/refresh", handle.RefreshCert)
	service.GET("/api/v1/cert", handle.GetCert)
	service.GET("/api/v1/domain/list", handle.GetCert)
}
